using IceCreamShopContracts.BindingModels;
using IceCreamShopContracts.BusinessLogicsContracts;
using IceCreamShopContracts.SearchModels;
using IceCreamShopContracts.StoragesContracts;
using IceCreamShopContracts.ViewModels;
using AbstractIceCreamShopDataModels.Models;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Reflection.Metadata;
using System.Xml.Linq;
using System.Reflection;

namespace IceCreamBusinessLogic.BusinessLogics
{
    public class ShopLogic : IShopLogic
    {
        private readonly ILogger _logger;
        private readonly IShopStorage _shopStorage;

        public ShopLogic(ILogger<ShopLogic> logger, IShopStorage shopStorage)
        {
            _logger = logger;
            _shopStorage = shopStorage;
        }

        public ShopViewModel? ReadElement(ShopSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. Shop Name:{0}.ID:{1}", model.Name, model.Id);
            var element = _shopStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }

        public List<ShopViewModel>? ReadList(ShopSearchModel? model)
        {
            _logger.LogInformation("ReadList. Shop Name:{0}.ID:{1} ", model?.Name, model?.Id);

            var list = (model == null) ? _shopStorage.GetFullList() : _shopStorage.GetFilteredList(model);

            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }

        public bool SupplyIceCreams(ShopSearchModel model, IIceCreamModel iceCream, int count)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }

            if (iceCream == null)
            {
                throw new ArgumentNullException(nameof(iceCream));
            }

            if (count <= 0)
            {
                throw new ArgumentNullException("Count of iceCreams in supply myst be more than 0", nameof(count));
            }

            var shopElement = _shopStorage.GetElement(model);
            if (shopElement == null)
            {
                _logger.LogWarning("Required shop element not found in storage");
                return false;
            }
            _logger.LogInformation("Shop element found. ID: {0}, Name: {1}", shopElement.Id, shopElement.Name);

            var countIcecreams = 0;
            foreach (var icecream in shopElement.ShopIceCreams)
            {
                countIcecreams += icecream.Value.Item2;
            }
            if (shopElement.IceCreamMaxCount - countIcecreams >= count)
            {
                if (shopElement.ShopIceCreams.TryGetValue(iceCream.Id, out var sameIcecream))
                {
                    shopElement.ShopIceCreams[iceCream.Id] = (iceCream, sameIcecream.Item2 + count);
                    _logger.LogInformation("Same iceCream found by supply. Added {0} of {1} in {2} shop", count, iceCream.IceCreamName, shopElement.Name);
                }
                else
                {
                    shopElement.ShopIceCreams[iceCream.Id] = (iceCream, count);
                    _logger.LogInformation("New iceCream added by supply. Added {0} of {1} in {2} shop", count, iceCream.IceCreamName, shopElement.Name);
                }
                _shopStorage.Update(new()
                {
                    Id = shopElement.Id,
                    Name = shopElement.Name,
                    Adress = shopElement.Adress,
                    OpeningDate = shopElement.OpeningDate,
                    ShopIceCreams = shopElement.ShopIceCreams,
                    IceCreamMaxCount = shopElement.IceCreamMaxCount
                });
            }
            else
            {
                _logger.LogWarning("Required shop is overflowed");
                return false;
            }
            return true;
        }

        public bool SellIceCreams(IIceCreamModel iceCream, int count)
        {
            return _shopStorage.SellIceCreams(iceCream, count);
        }

        public bool Create(ShopBindingModel model)
        {
            CheckModel(model);
            if (_shopStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }

            return true;
        }

        public bool Update(ShopBindingModel model)
        {
            CheckModel(model);
            if (_shopStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }

        public bool Delete(ShopBindingModel model)
        {
            CheckModel(model, false);
            if (_shopStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }

        private void CheckModel(ShopBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }

            if (!withParams)
            {
                return;
            }
            if (string.IsNullOrEmpty(model.Name))
            {
                throw new ArgumentNullException("Нет названия магазина!", nameof(model.Name));
            }
            _logger.LogInformation("Shop. Name: {0}, Adress: {1}, ID: {2}", model.Name, model.Adress, model.Id);
            var element = _shopStorage.GetElement(new ShopSearchModel
            {
                Name = model.Name
            });
            if (element != null && element.Id != model.Id)
            {
                throw new InvalidOperationException("Магазин с таким названием уже есть");
            }
        }
    }
}
