using IceCreamShopContracts.BindingModels;
using IceCreamShopContracts.SearchModels;
using IceCreamShopContracts.ViewModels;
using AbstractIceCreamShopDataModels.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace IceCreamShopContracts.BusinessLogicsContracts
{
    public interface IShopLogic
    {
        ShopViewModel? ReadElement(ShopSearchModel model);
        List<ShopViewModel>? ReadList(ShopSearchModel? model);
        bool Create(ShopBindingModel model);
        bool Update(ShopBindingModel model);
        bool Delete(ShopBindingModel model);
        bool SupplyIceCreams(ShopSearchModel model, IIceCreamModel iceCream, int count);
        bool SellIceCreams(IIceCreamModel iceCream, int count);
    }
}
